#include <stdlib.h>
#include <stdio.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER 0
#define AIRANGE       AD_B_10_V
#define BASETIME      40000000
#define SCANCOUNT     4000
#define ADCONVERTSRC  DAQ2K_AI_ADCONVSRC_Int
#define ADDLYTRIGSRC  DAQ2K_AI_DTSRC_AFI1
#define ADTRIGSRC     DAQ2K_AI_TRGSRC_ANA
#define ADTRIGMODE    DAQ2K_AI_TRGMOD_DELAY
#define ANATRIGCTRL   Above_High_Level|ADCATRIG
#define TIMERBASE     DAQ2K_IntTimeBase
#define BUFAUTORESET   1
#define SCAN_INTERVAL  400
#define DELAYCOUNT     10
#define ANATrigLevel_H 0x80
#define ANATrigLevel_L 0x80

short ai_buf[4000];
void show_channel_data(I16 *buf, U32 count, U16 startPos);

int main( void )
{
    I16 card, err, Id; 
    int card_num, card_type=0;
    U32 count=0, startPos;
    BOOLEAN fStop;

    setbuf( stdout, NULL );
    clrscr();

    printf("This program inputs %d scans from CH-0 of DAQ-2501/2502 in %d Hz.\n", SCANCOUNT, (int)(BASETIME/SCAN_INTERVAL));
    printf(" The Trigger source is set on FIRST AI channel in channel-gain queue\n" );
    printf(" Input the Delay Couter Source on AFI1 ( pin-21 ) \n\n\n" );

    printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
    scanf(" %d", &card_type);
    printf("Please input a card number: ");
    scanf(" %d", &card_num);
    if(card_type) card_type = DAQ_2502;
    else card_type = DAQ_2501;
    if ((card=D2K_Register_Card (card_type, (I16)card_num)) <0 ) {
	printf("Register_Card error=%d", card);
	return -1;
    }

    err = D2K_AIO_Config (card, TIMERBASE, ANATRIGCTRL, ANATrigLevel_H, ANATrigLevel_L);
    if (err != NoError) {
	printf("D2K_AIO_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    err = D2K_AI_CH_Config (card, CHANNELNUMBER, AIRANGE);
    if (err != NoError) {
	printf("D2K_AI_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    err = D2K_AI_Config (card, ADCONVERTSRC|ADDLYTRIGSRC, ADTRIGSRC|ADTRIGMODE, DELAYCOUNT, 0, 0, BUFAUTORESET);
    if (err != NoError) {
	printf("D2K_AI_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    err=D2K_AI_ContBufferSetup (card, ai_buf, SCANCOUNT, &Id);
    if (err != NoError) {
	printf("D2K_AI_ContBufferSetup error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    err = D2K_AI_ContReadChannel (card, CHANNELNUMBER, Id, SCANCOUNT, SCAN_INTERVAL*2, SCAN_INTERVAL, ASYNCH_OP);
    if (err != NoError) {
	printf("D2K_AI_ContReadChannel error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    printf("\n\n\nStart Data Conversion by External Analog Trigger Signal\n");
    do {
        D2K_AI_AsyncCheck(card, &fStop, &count);
    } while(!fStop);

    D2K_AI_AsyncClear(card, &startPos, &count);
    //Here to handle the data stored in ready buffer
    printf("\n\nTotal %ld input data.\n", count);
    printf("Press any key to show the data (at most 800 data)...\n");
    getch();

    show_channel_data(ai_buf, count, startPos);
    D2K_Release_Card(card);
    printf("\nPress ENTER to exit the program. "); getchar();

    return 0;
}

void show_channel_data( I16 *buf, U32 count, U16 startPos)
{
    int k;
    U32 i;

    printf(" >>>>>>>>>>>>>>> the first 100 valid scans  <<<<<<<<<<<<<<< \n");
    printf("CH0 :\n");
    for (k=0; k< (count/80) && k<10; k++) {
    	for( i=0+k*80; i<80+k*80 ; i++ ){
	    printf(" 0x%04x(%+4.2fV)  ", \
		ai_buf[(i+startPos)%SCANCOUNT]&0xffff, (F64)(ai_buf[(i+startPos)%SCANCOUNT])/ 32768 * 10.0);
	    if(i && ((i%4)==3))
		printf("\n");
	}
	if(k<(count/80-1) && k<9) {
	    printf("Press any key for the continued data...\n");
	    getch();
	}
    }
}
