/*********************************************************************
*
* Example program: 
*    CAIonePoint.c
*
* Description: 
*    Read one channel, one point at a time, immediately
*
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER 0
#define AIRANGE       AD_B_10_V
char cardStr[2][8] = {"DAQ2501", "DAQ2502"};

int main(void)
{
    I16 card, err; 
    int card_num, card_type=0;
    I16 chan_data;
    F64 chan_voltage;

    setbuf( stdout, NULL );
    clrscr();

    printf("This program inputs data from AD CH-0 of DAQ-2501/2502.\n");
    printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
    scanf(" %d", &card_type);
    if(card_type) card_type = DAQ_2502;
    else card_type = DAQ_2501;
    printf("Please input a card number: ");
    scanf(" %d", &card_num);

    if ((card=D2K_Register_Card (card_type, card_num)) <0 ) {
	printf("Register_Card error=%d", card);
	return -1;
    }
    else
   printf("Register_Card successful\n");
    err = D2K_AI_CH_Config (card, CHANNELNUMBER, AIRANGE);
    if (err != NoError) {
	printf("D2K_AI_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    do{
	clrscr();
        //get AI Hexadecimal value and transform to voltage 
	err = D2K_AI_ReadChannel (card, CHANNELNUMBER, &chan_data);
	if (err != NoError) {
	    printf("D2K_AI_ReadChannel error=%d", err);
	    D2K_Release_Card(card);
	    return -1;
	}
	chan_voltage = ((F64)(chan_data)) / 32768 * 10.0;;
        printf("Get AI Hexadecimal value and transform to voltage :\n"); 
	printf(" CH%d:\n", CHANNELNUMBER);
	printf("      input value :  0x%03x\n", (chan_data & 0xffff));
	printf("          voltage :  %.4f\n", chan_voltage);

        //get voltage directly from D2K_AI_VReadChannel:

	err = D2K_AI_VReadChannel (card, CHANNELNUMBER, &chan_voltage);
	if (err != NoError) {
	    printf("D2K_AI_VReadChannel error=%d", err);
	    D2K_Release_Card(card);
	    return -1;
	}
        printf("\nGet AI voltage directly:\n"); 
	printf(" CH%d:\n", CHANNELNUMBER);
	printf("          voltage :  %.4f\n", chan_voltage);
	printf("\n\n\n\n\n             press Any Key to stop \n");

	usleep(200000);

    } while(!kb_hit());

    D2K_Release_Card(card);

    return 0;
}
