#include <stdlib.h>
#include <stdio.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER 0
#define BASETIME      40000000
#define AIRANGE       AD_B_10_V
#define SCANCOUNT     4000
#define ADCONVERTSRC  DAQ2K_AI_ADCONVSRC_Int
#define ADTRIGSRC     DAQ2K_DA_TRGSRC_SOFT
#define ADTRIGMODE    DAQ2K_AI_TRGMOD_POST
#define BUFAUTORESET   1
#define SCAN_INTERVAL  4000
#define POSTCOUNT      0
#define SSI_SigCode    SSI_ADTRIG

short ai_buf[4000];
void show_channel_data(I16 *buf, U32 count, U16 startPos);

int main( void )
{
    I16 card, err, Id;
    int card_num, card_type = 0;
    BOOLEAN fStop=0;
    U32 count=0, startPos;

    setbuf( stdout, NULL );
    clrscr();

    printf("This program inputs %d scans from CH-0 of DAQ-2205/2206 in %d Hz.\n", SCANCOUNT, (int)(BASETIME/SCAN_INTERVAL));
    printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
    scanf(" %d", &card_type);
    printf("Please input a card number: ");
    scanf(" %d", &card_num);
    if(card_type == 1) card_type = DAQ_2502;
    else card_type = DAQ_2501;
    if ((card=D2K_Register_Card (card_type, (I16)card_num)) <0 ) {
	printf("Register_Card error=%d", card);
	return -1;
    }

    err = D2K_AI_CH_Config (card, CHANNELNUMBER, AIRANGE);
    if (err!=NoError) {
	printf("D2K_AI_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    err = D2K_AI_Config (card, ADCONVERTSRC, ADTRIGSRC|ADTRIGMODE, POSTCOUNT, 0, 0, BUFAUTORESET);
    if (err!=NoError) {
	printf("D2K_AI_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    err = D2K_SSI_SourceConn (card, SSI_SigCode);
    if (err!=NoError) {
	printf("D2K_SSI_SourceConn error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    err = D2K_AI_ContBufferSetup (card, ai_buf, SCANCOUNT, &Id);
    if (err!=NoError) {
	printf("D2K_AI_ContBufferSetup error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    err = D2K_AI_ContReadChannel (card, CHANNELNUMBER, Id, SCANCOUNT, SCAN_INTERVAL, SCAN_INTERVAL, ASYNCH_OP);
    if (err!=NoError) {
	printf("D2K_AI_ContReadChannel error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    do {
        D2K_AI_AsyncCheck(card, &fStop, &count);
    } while(!fStop);

    D2K_AI_AsyncClear(card, &startPos, &count);
    //Here to handle the data stored in ready buffer
    printf("\n\nTotal %ld input data.\n", count);
    printf("Press any key to show the data (at most 800 data)...\n");
    getch();

    show_channel_data(ai_buf, count, startPos);
    D2K_Release_Card(card);
    printf("\nPress ENTER to exit the program. "); getchar();

    return 0;
}

void show_channel_data( I16 *buf, U32 count, U16 startPos)
{
    int k;
    U32 i;

    printf(" >>>>>>>>>>>>>>> the first 100 valid scans  <<<<<<<<<<<<<<< \n");
    printf("CH0 :\n");
    for (k=0; k< (count/80) && k<10; k++) {
	for( i=(k*80); i<(k*80 +80) ; i++ ){
	    printf(" 0x%04x(%+4.2fV)  ", \
		ai_buf[(i+startPos)%SCANCOUNT]&0xffff, (F64)(ai_buf[(i+startPos)%SCANCOUNT]) / 32768 * 10.0);
	    if(i && ((i%4)==3))
		printf("\n");
	}
	if(k<(count/80-1) && k<9) {
	    printf("Press any key for the continued data...\n");
	    getch();
	}
    }
}

