/*********************************************************************
*
* Example program: 
*    CAIonePoint.c
*
* Description: 
*    Read 4 channels, one point from each channel at a time
*
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER All_Channels
#define CHANNELCOUNT  4
#define AIRANGE       AD_B_10_V

int main(void)
{
    I16 card, err, i; 
    int card_num, card_type=0;
    I16 chan_data[4];
    F64 chan_voltage[4];
    U16 chans[16];

    setbuf( stdout, NULL );
    clrscr();

    printf("This program inputs data from CH-0 to CH-3 of DAQ-2501/2502.\n");
    printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
    scanf(" %d", &card_type);
    if(card_type) card_type = DAQ_2502;
    else card_type = DAQ_2501;

    printf("Please input a card number: ");
    scanf(" %d", &card_num);

    if ((card=D2K_Register_Card (card_type, (I16)card_num)) <0 ) {
	printf("Register_Card error=%d", card);
	return -1;
    }

    err = D2K_AI_CH_Config (card, CHANNELNUMBER, AIRANGE);
    if (err!=NoError) {
	printf("D2K_AI_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    for (i=0; i<CHANNELCOUNT; i++) {
       chans[i] = i;  //ascending
    }
    do{
	clrscr();
        //get AI Hexadecimal values and transform to voltages 
	err = D2K_AI_ScanReadChannels(card, CHANNELCOUNT, chans, chan_data);
	if (err!=NoError) {
	    printf("D2K_AI_SimuReadChannel error=%d", err);
	    D2K_Release_Card(card);
	    return -1;
	}
	for (i=0; i<CHANNELCOUNT; i++) 
	    chan_voltage[i] = (F64)(chan_data[i]) / 32768 * 10.0;
	printf("                Ch0       Ch1       Ch2       Ch3\n");
	printf(" input value :  %04x      %04x      %04x      %04x\n", \
	    chan_data[0]&0xffff, chan_data[1]&0xffff, chan_data[2]&0xffff, chan_data[3]&0xffff);
	printf("     voltage :  %.2f      %.2f      %.2f      %.2f\n\n", \
	    chan_voltage[0], chan_voltage[1], chan_voltage[2], chan_voltage[3]);
        printf("\n\n\n\n\n                    press and hold ANY key to stop \n");

	usleep(200000);
    } while(!kb_hit());

    D2K_Release_Card(card);

    return 0;

}

