#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include "conio.h"
#include "d2kdask.h"

///////////////////////////////////////////////////////////////////////////////
#define PI          3.14159
#define SIZE 20000
U16 AO_buf[SIZE];
U32 data_size = SIZE;

struct timeval tv_start, tv_end;
struct timezone tz;


#pragma pack(push,1)

typedef struct _D2K_MODULE_PARAM
{
		short	card_number;			
		short	card_type;	
		unsigned short ch;
		unsigned long   scancount;
		unsigned long   scan_interval;
}
D2K_MODULE_PARAM;

#pragma pack(pop)

D2K_MODULE_PARAM param;

void evt_handler(int type, void* arg) {
   gettimeofday( &tv_end, &tz );
   printf(" End time for DMA operation is: %ld.%06ld sec\n", tv_end.tv_sec, tv_end.tv_usec);
   printf(" A end: evt_%hd|cardtype_%hd|cardid_%hd\n", type, ((D2K_MODULE_PARAM *) arg)->card_type, ((D2K_MODULE_PARAM *) arg)->card_number);
   printf(" Press any key to end data output \n" );
}

int main(int argc, char* argv[])
{
   I16 card = -1, err, i; 
   int card_num, card_type;
   U32 count;
   U16 DaId, da_ch = 0; //DA channel 0
   U8 bStopped;

   setbuf( stdout, NULL );
   clrscr();

   printf("This program outputs data from AO CH-0 of DAQ-2501/2502 in %d Hz.\nThe size of buffer is %ld. The iteration is 2.\n\nPress any key to start the operation.\n", 100000, data_size );

   printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
   scanf(" %d", &card_type);
   printf("Please input a card number: ");
   scanf(" %d", &card_num);
   if(card_type) card_type = DAQ_2502;
   else card_type = DAQ_2501;
   if((card = D2K_Register_Card(card_type, card_num))<0) {
	 printf("Error %d trying to open daq2501/2502\n", card);
	 exit(1);
   }
   
   err = D2K_AO_CH_Config (card, 0, DAQ2K_DA_BiPolar, DAQ2K_DA_Int_REF, 10.0);
   if( err != NoError ){
	printf(" D2K_AO_CH_Config Error %d\n" , err );
	D2K_Release_Card(card);
	return -1;
   }
   err = D2K_AO_Group_Setup (card, DA_Group_A, 1, &da_ch); //Group A
   if( err != NoError ){
	printf(" D2K_AO_Group_Setup Error %d\n" , err );
	D2K_Release_Card(card);
	return -1;
   }
   err = D2K_AO_Config (card, DA_Group_A, DAQ2K_DA_TRGMOD_POST, 0, 0, 0, 0);
   if( err != NoError ){
	printf(" D2K_AO_Config Error %d\n" , err );
	D2K_Release_Card(card);
	return -1;
   }

   for(i=0; i<SIZE; i++) {
	AO_buf[i]= (U16) ((sin((double)i/1000*PI)*0x7ff)+0x800);
   }
#if 0   
   err = D2K_AO_EventCallBack (card, 1, DAQEnd, signal_handler);
   if( err != NoError ){
       printf("D2K_AO_EventCallBack failed error=%d", err);
   }
#endif
   err = D2K_AO_ContBufferSetup (card, AO_buf, data_size, &DaId);
   if( err != NoError ){
	printf(" D2K_AO_ContBufferSetup Error %d\n" , err );
	D2K_Release_Card(card);
	return -1;
   }
   //32-bit width of data: scan count is the half of data_size 
	//set callback param
	param.card_number=card;
	param.card_type=card_type;
	param.ch=0;
	param.scan_interval=400;
	param.scancount=data_size/2;

   // setup signal handler
   D2K_AO_EventCallBackEx (card, 1, DAQEnd, evt_handler, (void* ) &param); 
    gettimeofday( &tv_start, &tz );
    printf(" Start time for DMA operation is: %ld.%06ld sec\n", tv_start.tv_sec, tv_start.tv_usec);

   err = D2K_AO_Group_WFM_Start (card, DA_Group_A, DaId, 0, data_size/2, 2, 400, 1);
   if(err!=NoError){
	printf("D2K_AO_Group_WFM_Start Error %d \n", err);
	D2K_AO_ContBufferReset (card);
	D2K_Release_Card(card);
	return -1;
   }
   getch();
   D2K_AO_Group_WFM_AsyncClear(card, DA_Group_A, &count, 0);
   //reset signal
   err = D2K_AO_EventCallBackEx (card, 0, DAQEnd, NULL, NULL);
   if( err != NoError ){
       printf("D2K_AO_EventCallBackEx failed error=%d", err);
   }
   err = D2K_AO_ContBufferReset (card);
   if( err != NoError ){
       printf("D2K_AO_ContBufferReset failed error=%d", err);
   }

   D2K_Release_Card(card);

   return 0;
}// main
