/*********************************************************************
*
* Example program: 
*    AOonePoint.c
*
* Description: 
*    Output one channel, one point at a time, immediately 
*
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER 0
#define DAPOLARITY	  DAQ2K_DA_BiPolar
#define DAREFERENCE	  DAQ2K_DA_Int_REF
#define DAREFERENCE_VOLTAGE	  10.0

int main( void )
{
    I16 card, err; 
    int card_num=0, card_type;
    U16 DA_value, da_ch=CHANNELNUMBER;
    F32 out_voltage;
    BOOLEAN bstop = FALSE;
    char cont_input;

    setbuf( stdout, NULL );
    clrscr();

    /* For best results, look at the output with a scope. */
    printf("This program outputs data to DA0.\n");
    printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");

    scanf(" %d", &card_type);
    printf("Please input a card number: ");
    scanf(" %d", &card_num);

    if(card_type == 1) card_type = DAQ_2502;
    else card_type = DAQ_2501;

    if ((card=D2K_Register_Card (card_type, card_num)) <0 ) {
	printf("Register_Card error=%d", card);
	return -1;
    }

    err = D2K_AO_CH_Config (card, CHANNELNUMBER, DAPOLARITY, DAREFERENCE, (F64)DAREFERENCE_VOLTAGE);
    if (err != NoError) {
	printf("D2K_AO_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    err = D2K_AO_Group_Setup (card, DA_Group_A, 1, &da_ch);
    if (err != NoError) {
	printf("D2K_AO_Group_Setup error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    printf("\n Please input output Voltage value in [%s]:", "-10v ~ +10v");
    scanf(" %f", &out_voltage );

    do{
        //transform voltage to Hexadecimal value
	DA_value = (U16)( (out_voltage +10) / 20 * 4095 );
        err = D2K_AO_Group_Update (card,  DA_Group_A, &DA_value);
	//You can pass voltage to D2K_AO_VWriteChannel:
	
	if (err != NoError) {
	    printf("D2K_AO_Group_Update error=%d", err);
	    D2K_Release_Card(card);
	    return -1;
	}

	printf(" Output Voltage : %f V (0x%x) ", out_voltage, DA_value);
	printf("\n");
	usleep(500000);

	printf("                 press (C)ontinue or Quit :");
	cont_input=getch();

	if( cont_input != 'C' && cont_input != 'c' ){
	    bstop = TRUE;
	} else {
	    bstop = FALSE;
	    printf("\n\n Please input output Voltage value in [%s]:", "-10v ~ +10v");
	    scanf(" %f", &out_voltage );
	}
    }while( !bstop );

    D2K_Release_Card(card);

    putchar('\n');
    return 0;
}

