#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include "conio.h"
#include "d2kdask.h"

///////////////////////////////////////////////////////////////////////////////
#define PI          3.14159
#define SIZE 4096
U16 AO_buf[SIZE];
U32 data_size = SIZE;

int main(int argc, char* argv[])
{
    I16 card = -1, err, i;
    int card_num, card_type=0;
    U32 count;
    U16 DaId, da_ch[2]={0, 4};
    U8 bStopped;
    struct timeval tv_start, tv_end;
    struct timezone tz;

    setbuf( stdout, NULL );
    clrscr();

    tz.tz_minuteswest = 0;
    tz.tz_dsttime = 0;

    printf("This program outputs data from AO CHs in %d Hz.\nThe size of buffer is %ld. The iteration is 2.\n\nPress any key to start the operation.\n\n",
           100000, data_size);
    printf("This program demonstrates Double Buffered DA\n\n");

    printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
    scanf(" %d", &card_type);
    printf("Please input a card number: ");
    scanf(" %d", &card_num);

    if(card_type == 1) card_type = DAQ_2502;
    else card_type = DAQ_2501;

    if((card = D2K_Register_Card(card_type, (I16)card_num))<0) {
	printf("Error %d trying to open daq2501/2502\n", card);
	return -1;
    }

   err = D2K_AO_CH_Config (card, 0, DAQ2K_DA_BiPolar, DAQ2K_DA_Int_REF, 10.0);
   if(err!=NoError){
	printf("D2K_AO_CH_Config Error %d \n", err);
	D2K_Release_Card(card);
	return -1;
   }
   err = D2K_AO_Group_Setup (card, DA_Group_A, 1, &da_ch[0]);
   if(err!=NoError){
	printf("D2K_AO_Group_Setup Error %d \n", err);
	D2K_Release_Card(card);
	return -1;
   }

   //Delay timer source: AFI0
   //Trigger source: EXTWFTRG_A

   err = D2K_AO_Config (card, DA_Group_A|DAQ2K_DA_TDSRC_AFI0, DAQ2K_DA_TRGMOD_DELAY|DAQ2K_DA_ReTrigEn|DAQ2K_DA_TRGSRC_ExtD|DAQ2K_DA_TrgPositive, 2, 100, 0, 0);
   if(err!=NoError){
	printf("D2K_AO_Config Error %d \n", err);
	D2K_Release_Card(card);
	return -1;
   }

   for(i=0; i<SIZE; i++) {
	AO_buf[i]= (sin((double)i/512*PI)*0x7ff)+0x800;
   }

   err = D2K_AO_ContBufferSetup (card, AO_buf, data_size, &DaId);
   if(err!=NoError){
	printf("D2K_AO_ContBufferSetup Error %d \n", err);
	D2K_Release_Card(card);
	return -1;
   }

   err = D2K_AO_Group_FIFOLoad (card, DA_Group_A, DaId, data_size);
   if(err!=NoError){
	printf("D2K_AO_Group_FIFOLoad Error %d \n", err);
	D2K_AO_ContBufferReset (card);
	D2K_Release_Card(card);
	return -1;
   }

   gettimeofday( &tv_start, &tz );
   printf(" Start time for DMA operation is: %ld.%06ld sec\n", tv_start.tv_sec, tv_start.tv_usec);

   err = D2K_AO_Group_WFM_Start (card, DA_Group_A, BufferNotUsed, 0, 0, 10, 400, 1);
   if(err!=NoError){
	printf("D2K_AO_Group_WFM_Start Error %d \n", err);
	D2K_AO_ContBufferReset (card);
	D2K_Release_Card(card);
	return -1;
   }

   do {
	D2K_AO_Group_WFM_AsyncCheck(card, DA_Group_A, &bStopped, &count);
   } while (bStopped != 1);

   D2K_AO_Group_WFM_AsyncClear(card, DA_Group_A, &count, 0);
   D2K_AO_ContBufferReset (card);

   gettimeofday( &tv_end, &tz );
   printf(" End time for DMA operation is: %ld.%06ld sec\n", tv_end.tv_sec, tv_end.tv_usec);

   getchar();
   D2K_Release_Card(card);

   return 0;
}// main
