#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include "conio.h"
#include "d2kdask.h"

///////////////////////////////////////////////////////////////////////////////
#define PI          3.14159
#define SIZE0 8192
U16 AO_buf[SIZE0];
U16 AO_buf2[SIZE0];
U32 data_size = SIZE0;

int main(int argc, char* argv[])
{
   I16 card = -1, err, i, chcnt = 4;
   int card_num, card_type;
   U32 count;
   U16 DaId, DaId2, da_ch[8];
   U8 bStopped;
   struct timeval tv_start, tv_end;
   struct timezone tz;


   setbuf( stdout, NULL );
   clrscr();

   tz.tz_minuteswest = 0;
   tz.tz_dsttime = 0;

   printf("This program outputs data from AO CHs of DAQ-2501/2502 in %d Hz.\nThe size of buffer is %ld. The iteration is 10.\n\nPress any key to start the operation.\n", 100000, data_size );

    printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
    scanf(" %d", &card_type);
    printf("Please input a card number: ");
    scanf(" %d", &card_num);
    if(card_type == 1) card_type = DAQ_2502;
    else card_type = DAQ_2501;
   if((card = D2K_Register_Card(card_type, (I16)card_num))<0) {
	printf("Error %d trying to open daq2010\n", card);
	return -1;
   }

   err = D2K_AO_CH_Config (card, All_Channels, DAQ2K_DA_BiPolar, DAQ2K_DA_Int_REF, 10.0);
   if (err!=NoError) {
	printf("D2K_AO_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
   }

   chcnt = (card_type==DAQ_2502)? 8 : 4;
   for(i=0; i<chcnt; i++)
	da_ch[i] = i;

   err = D2K_AO_Group_Setup (card, (card_type==DAQ_2502)?DA_Group_AB:DA_Group_A, chcnt, da_ch);
   if (err!=NoError) {
	printf("D2K_AO_Group_Setup error=%d", err);
	D2K_Release_Card(card);
	return -1;
   }

   for(i=0; i<SIZE0; i++) {
	AO_buf[i]= (U16) ((sin((double)i/4096*PI)*0x7ff)+0x800); //GroupA 
   }

   err = D2K_AO_ContBufferSetup (card, AO_buf, data_size, &DaId);
   if (err!=NoError) {
	printf("D2K_AO_ContBufferSetup error=%d", err);
	D2K_Release_Card(card);
	return -1;
   }
   err = D2K_AO_Group_FIFOLoad (card, DA_Group_A, DaId, data_size);
   if(err!=NoError){
	printf("D2K_AO_Group_WFM_Start Error %d \n", err);
	D2K_AO_ContBufferReset (card);
	D2K_Release_Card(card);
	return -1;
   }

   switch(card_type) {
	case DAQ_2501:
	    D2K_AO_Config (card, DA_Group_A|DAQ2K_DA_WRSRC_Int, DAQ2K_DA_TRGMOD_POST, 0, 0, 0, 0);

	    gettimeofday( &tv_start, &tz );
	    err = D2K_AO_Group_WFM_Start (card, DA_Group_A, BufferNotUsed, 0, 0, 10, 400, 1);
	    if(err!=NoError){
		printf("D2K_AO_Group_WFM_Start Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }

	    do {
		D2K_AO_Group_WFM_AsyncCheck(card, DA_Group_A, &bStopped, &count);
	    } while (bStopped != 1);
	    D2K_AO_Group_WFM_AsyncClear(card, DA_Group_A, &count, 0);
	    break;

	case DAQ_2502:
	    for(i=0; i<SIZE0; i++) {
		AO_buf2[i] = (0x800+i%8192) & 0xfff; //Group B
	    }

	    err = D2K_AO_ContBufferSetup (card, AO_buf2, data_size, &DaId2);
   	    if(err!=NoError){
		printf("D2K_AO_ContBufferSetup#1 Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
	    err = D2K_AO_Group_FIFOLoad (card, DA_Group_B, DaId2, data_size);  
   	    if(err!=NoError){
		printf("D2K_AO_Group_FIFOLoad Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
	    err = D2K_AO_Config (card, DA_Group_AB|DAQ2K_DA_WRSRC_Int, DAQ2K_DA_TRGMOD_POST, 0, 0, 0, 0);
   	    if(err!=NoError){
		printf("D2K_AO_Config Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
	    gettimeofday( &tv_start, &tz );
	    err = D2K_AO_Group_WFM_Start (card, DA_Group_AB, BufferNotUsed, 0, 0, 10, 400, 1);
	    if(err!=NoError){
		printf("D2K_AO_Group_WFM_Start Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
	    do {
		D2K_AO_Group_WFM_AsyncCheck(card, DA_Group_AB, &bStopped, &count);
	    } while (bStopped != 3);
	    D2K_AO_Group_WFM_AsyncClear(card, DA_Group_AB, &count, 0);
	    break;
    }
    gettimeofday( &tv_end, &tz );

    D2K_AO_ContBufferReset (card);
    printf(" The time for DMA operation is: %ld usec\n", (tv_end.tv_sec - tv_start.tv_sec)*1000000L + (tv_end.tv_usec - tv_start.tv_usec ) );
    getchar();

    D2K_Release_Card(card);

    return 0;
}// main
