/*********************************************************************
*
* Example program: 
*    AOsimuUpdate.c
*
* Description: 
*    Output two channels, one point for each channel at a time, immediately 
*
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER All_Channels
#define DAPOLARITY	  DAQ2K_DA_BiPolar
#define DAREFERENCE	  DAQ2K_DA_Int_REF
#define DAREFERENCE_VOLTAGE	  10.0

int main(void)
{
    I16 card, err, chcnt = 4, i=0;
    int card_num, card_type;
    F32 out_voltage;
    BOOLEAN bstop = FALSE;
    char cont_input;
    I16 outVal[8], group;
    U16 da_ch[8];

    setbuf( stdout, NULL );
    clrscr();

    /* For best results, look at the output with a scope. */
    printf("This program outputs data to GroupA (and B)\n");
    printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
    scanf(" %d", &card_type);
    printf("Please input a card number: ");
    scanf(" %d", &card_num);
    if(card_type == 1) card_type = DAQ_2502;
    else card_type = DAQ_2501;

    if ((card=D2K_Register_Card (card_type, (I16)card_num)) <0 ) {
	printf("Register_Card error=%d", card);
	return -1;
    }
    err = D2K_AO_CH_Config (card, CHANNELNUMBER, DAPOLARITY, DAREFERENCE, (F64)DAREFERENCE_VOLTAGE);
    if (err != NoError) {
	printf("D2K_AO_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    chcnt = (card_type==DAQ_2502)? 8 : 4;
    for(i=0; i<chcnt; i++)
	da_ch[i] = i;

    group = (card_type==DAQ_2502)? DA_Group_AB : DA_Group_A;
    err = D2K_AO_Group_Setup (card, group, chcnt, da_ch);
    if (err != NoError) {
	printf("D2K_AO_Group_Setup error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    printf("\n Please input output Voltage value in [%s]:", "-10v ~ +10v");
    scanf(" %f", &out_voltage );

    do{
        //transform voltage to Hexadecimal value
	for(i=0; i<chcnt; i++)
	    outVal[i]= (U16)( (out_voltage +10) / 20 * 4095 );

        err = D2K_AO_Group_Update (card,  group, outVal);
	if (err!=NoError) {
	    printf("D2K_AO_Group_Update error=%d", err);
	    D2K_Release_Card(card);
	    return -1;
	}
	printf(" DA0-DA%d Output Voltage : %f V (0x%x) ", chcnt-1, out_voltage, outVal[0]);
	printf("\n");
	usleep(500000);

	printf("                 press (C)ontinue or Quit :");
	cont_input=getch();

	if( cont_input != 'C' && cont_input != 'c' ){
	    bstop = TRUE;
	} else {
	    bstop = FALSE;
	    printf("\n\n Please input output Voltage value in [%s]:", "-10v ~ +10v");
	    scanf(" %f", &out_voltage );
	}
    }while( !bstop );

    D2K_Release_Card(card);

    putchar('\n');

    return 0;
}
