#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include "conio.h"
#include "d2kdask.h"

///////////////////////////////////////////////////////////////////////////////
#define PI          3.14159
#define ONE_CH_SIZE 4096
U16 pattern1[ONE_CH_SIZE];
U16 pattern2[ONE_CH_SIZE];
U16 pattern3[ONE_CH_SIZE];
U16 pattern4[ONE_CH_SIZE];
U16 AO_buf[4*ONE_CH_SIZE];

int main(int argc, char* argv[])
{
   I16 card = -1, err, i, chcnt = 4;
   int card_num, card_type;
   U32 count;
   U16 DaId, da_ch[4];
   void* wfmPtr[4]; 
   U8 bStopped;
   struct timeval tv_start, tv_end;
   struct timezone tz;

   setbuf( stdout, NULL );
   clrscr();
   tz.tz_minuteswest = 0;
   tz.tz_dsttime = 0;

   printf("This program outputs data from AO CHs of DAQ-2501/2502 in %d Hz.\nThe size of each channel is %d. The iteration is 10.\n\nPress any key to start the operation.\n", 100000, ONE_CH_SIZE );

   printf("Card Type: (0) DAQ_2501 or (1) DAQ2502 ? ");
   scanf(" %d", &card_type);
   printf("Please input a card number: ");
   scanf(" %d", &card_num);

   if(card_type ==1) card_type = DAQ_2502;
   else card_type = DAQ_2501;
   if((card = D2K_Register_Card(card_type, (I16)card_num))<0) {
	printf("Error %d trying to open daq2501/2502\n", card);
	return -1;
   }
   err = D2K_AO_CH_Config (card, All_Channels, DAQ2K_DA_BiPolar, DAQ2K_DA_Int_REF, 10.0);
   if (err != NoError) {
	printf("D2K_AO_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
   }
   chcnt = (card_type==DAQ_2502)? 4 : 2; 
   da_ch[0] = 0;
   da_ch[1] = 1;
   da_ch[2] = 4;
   da_ch[3] = 5;
   if(card_type==DAQ_2502)
	err = D2K_AO_Group_Setup (card, DA_Group_AB, chcnt, da_ch);
   else
	err = D2K_AO_Group_Setup (card, DA_Group_A, chcnt, da_ch);
   if (err != NoError) {
	printf("D2K_AO_Group_Setup error=%d", err);
	D2K_Release_Card(card);
	return -1;
   }

   //Generate the waveform data for each DA channel
   //SINEWAVE for channel 0
   for (i=0;i<ONE_CH_SIZE;i++) 
	pattern1[i] = (I16)(sin((double)i/512*PI)*0x7ff)+0x800; 
   //TRIANGLE for channel 1
   for (i=0;i<ONE_CH_SIZE;i++) {
	if(i<2048)
	    pattern2[i] = (0x800+i%2048) & 0xfff;
	else 
	    pattern2[i] = (0x800+2047-(i%2048)) & 0xfff;
   }
   wfmPtr[0] = pattern1;
   wfmPtr[1] = pattern2;

   if(card_type==DAQ_2502) {
	//SQUARE Wave for channel 4
	for (i=0;i<ONE_CH_SIZE;i++) {
	    if(i<2048)
		pattern3[i] = (0) & 0xfff;
	    else 
		pattern3[i] = (4095) & 0xfff;
	}
	//SQUARE Wave for channel 5
	for (i=0;i<ONE_CH_SIZE;i++) 
	    pattern4[i] = (I16)(sin((double)i/2048*PI)*0x7ff)+0x800;

	wfmPtr[2] = pattern3;
	wfmPtr[3] = pattern4;
   }
   
   switch(card_type) {
	case DAQ_2501:
	    err = D2K_AO_ContBufferComposeAll (card, DA_Group_A, ONE_CH_SIZE, AO_buf, wfmPtr, 1);   
	    if(err!=NoError){
		printf("D2K_AO_ContBufferComposeAll Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		return -1;
	    }
	    err = D2K_AO_ContBufferSetup (card, AO_buf, ONE_CH_SIZE*chcnt, &DaId);
	    if(err!=NoError){
		printf("D2K_AO_ContBuffersetup Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		return -1;
	    }
	    err = D2K_AO_Group_FIFOLoad (card, DA_Group_A, DaId, ONE_CH_SIZE*chcnt);
	    if(err!=NoError){
		printf("D2K_AO_Group_FIFOLoad Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
	    err = D2K_AO_Config (card, DA_Group_A|DAQ2K_DA_WRSRC_Int, DAQ2K_DA_TRGMOD_POST, 0, 0, 0, 0);
	    if(err!=NoError){
		printf("D2K_AO_Config Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
            gettimeofday( &tv_start, &tz );
	    err = D2K_AO_Group_WFM_Start (card, DA_Group_A, BufferNotUsed, 0, 0, 10, 400, 1);
	    if(err!=NoError){
		printf("D2K_AO_Group_WFM_Start Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
	    do {
		D2K_AO_Group_WFM_AsyncCheck(card, DA_Group_A, &bStopped, &count);
	    } while (bStopped != 1);		

	    D2K_AO_Group_WFM_AsyncClear(card, DA_Group_A, &count, 0);
	    break;

   case DAQ_2502:
	    err = D2K_AO_ContBufferComposeAll (card, DA_Group_AB, ONE_CH_SIZE, AO_buf, wfmPtr, 1);   
	    if(err!=NoError){
		printf("D2K_AO_ContBufferComposeAll Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		return -1;
	    }
	    err = D2K_AO_ContBufferSetup (card, AO_buf,ONE_CH_SIZE*chcnt, &DaId); //4 channels
	    if(err!=NoError){
		printf("D2K_AO_ContBuffersetup Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		return -1;
	    }
	    err = D2K_AO_Group_FIFOLoad (card, DA_Group_AB, DaId, ONE_CH_SIZE*chcnt); 
	    if(err!=NoError){
		printf("D2K_AO_Group_FIFOLoad Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
	    err = D2K_AO_Config (card, DA_Group_AB|DAQ2K_DA_WRSRC_Int, DAQ2K_DA_TRGMOD_POST, 0, 0, 0, 0);
	    if(err!=NoError){
		printf("D2K_AO_Config Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }
            gettimeofday( &tv_start, &tz );
	    err = D2K_AO_Group_WFM_Start (card, DA_Group_AB, BufferNotUsed, 0, 0, 10, 400, 1);
	    if(err!=NoError){
		printf("D2K_AO_Group_WFM_Start Error %d \n", err);
		D2K_AO_ContBufferReset (card);
		D2K_Release_Card(card);
		return -1;
	    }

	    do {
		D2K_AO_Group_WFM_AsyncCheck(card, DA_Group_AB, &bStopped, &count);
	    } while (bStopped != 3);
	    D2K_AO_Group_WFM_AsyncClear(card, DA_Group_AB, &count, 0);
	    break;
   }
   gettimeofday( &tv_end, &tz );
   D2K_AO_ContBufferReset (card);
   printf(" The time for DMA operation is: %ld usec\n", (tv_end.tv_sec  - tv_start.tv_sec)*1000000L + (tv_end.tv_usec - tv_start.tv_usec ) );
   D2K_Release_Card(card);

   return 0;
}// main
