#include <stdlib.h>
#include <stdio.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER 0
#define BASETIME      40000000
#define AIRANGE       AD_B_10_V
#define SCANCOUNT     4000
#define ADCONVERTSRC  DAQ2K_AI_ADCONVSRC_Int
#define ADTRIGSRC     DAQ2K_AI_TRGSRC_ExtD
#define ADTRIGMODE    DAQ2K_AI_TRGMOD_DELAY
#define ADTRIGPOL     DAQ2K_AI_TrgPositive
#define BUFAUTORESET  1
#define SCAN_INTERVAL 4000
#define DELAYCOUNT    1000
#define DELAYMODE     DAQ2K_AI_Dly1InTimebase

short ai_buf[4000];
short ai_buf2[4000];
FILE *svfile;
char cardStr[2][8] = {"DAQ2501", "DAQ2502"};

void write_to_file( I16 *buf, U32 write_count )
{
  U32 i;
	
  for(i=0; i<write_count; i++) {
    fprintf( svfile, "0x%04x(%4.2fV)\n", buf[i] & 0xffff, (F64)(buf[i]) / 32768 * 10.0);
  }
}

int main( void )
{
    I16 card, err, Id, tmpId = 0; 
    int card_num, card_type = 0, fok = 0;
    BOOLEAN halfReady, fStop;
    U32 count=0, count1, startPos;

    setbuf( stdout, NULL );
    clrscr();
    printf("This program inputs %d scans from CH-0 of DAQ-2205/2206 in %d Hz.\n", SCANCOUNT, (int)(BASETIME/SCAN_INTERVAL));
    printf("Card Type: (0) DAQ-2501 or (1) DAQ-2502 ? ");
    scanf(" %d", &card_type);
    printf("Please input a card number: ");
    scanf(" %d", &card_num);
    if( card_type == 1 ) card_type = DAQ_2502;
    else card_type = DAQ_2501;

    if ((card=D2K_Register_Card (card_type, (I16)card_num)) <0 ) {
        printf("Register_Card error=%d", card);
	return -1;
    }

    printf("Save data to a file (DmaData.dat): Yes(1) or No(0)? ");
    scanf(" %d", &fok);

    err = D2K_AI_CH_Config (card, CHANNELNUMBER, AIRANGE);
    if (err != NoError) {
	printf("D2K_AI_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    err = D2K_AI_Config (card, ADCONVERTSRC, ADTRIGSRC|ADTRIGMODE|ADTRIGPOL|DELAYMODE, DELAYCOUNT, 0, 0, BUFAUTORESET);
    if (err != NoError) {
	printf("D2K_AI_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    err=D2K_AI_AsyncDblBufferMode (card, 1);
    if (err != NoError) {
	printf("D2K_AI_AsyncDblBufferMode error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    if( fok == 1) {
	svfile = fopen("DmaData.dat", "w");
	fprintf( svfile, "CH%d :\n", CHANNELNUMBER);
    }
    err = D2K_AI_ContBufferSetup (card, ai_buf, SCANCOUNT, &Id);
    if (err != NoError) {
	printf("D2K_AI_ContBufferSetup#0 error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    err = D2K_AI_ContBufferSetup (card, ai_buf2, SCANCOUNT, &Id);
    if (err != NoError) {
	printf("D2K_AI_ContBufferSetup#1 error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }
    err = D2K_AI_ContReadChannel (card, CHANNELNUMBER, Id, SCANCOUNT, SCAN_INTERVAL, SCAN_INTERVAL, ASYNCH_OP);
    if (err!=0) {
	printf("D2K_AI_ContReadChannel error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    printf("\n\n\nStart Data Conversion by External Trigger Signal on AFI0\n Press and hold ANY key to stop Opeartion.\n");
    printf("\n\nData count : \n");

    do {
        do {
             D2K_AI_AsyncDblBufferHalfReady(card, &halfReady, &fStop);
	} while (!halfReady && !fStop);

	//Here to handle the data stored in ready buffer
	count += (SCANCOUNT);
        printf("%ld\r", count);
	if( fok == 1 )
           write_to_file( tmpId?ai_buf2:ai_buf, SCANCOUNT );

	tmpId = (tmpId + 1) % 2;
    } while(!kb_hit());

    D2K_AI_AsyncClear(card, &startPos, &count1);

    if(fok == 1)
	fclose(svfile);

    printf("\n\nTotal %ld input data.\n", count);

    D2K_Release_Card(card);
    printf("\nPress ENTER to exit the program. "); getchar();

    return 0;
}

