#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include "conio.h"
#include "d2kdask.h"

#define MAX_CHANNELS  4
#define SCANCOUNT     2
#define AIRANGE       AD_B_10_V
#define BASETIME      40000000
#define BUFAUTORESET    1
#define SAMPLE_INTERVAL 1000
#define SCAN_INTERVAL   SAMPLE_INTERVAL * MAX_CHANNELS

#define CardNum   12

U16 CardType[CardNum] = {DAQ_2005, DAQ_2006, DAQ_2010, DAQ_2016, DAQ_2204, DAQ_2205, DAQ_2206, DAQ_2208, DAQ_2213, DAQ_2214, DAQ_2501, DAQ_2502};

void RetrieveVoltagesFromDoubleBuffer(U16 card, unsigned short* pRawSampleBuffer)
{
   unsigned int channel = 0;
   double ADCVoltages[MAX_CHANNELS];
   for(; channel < MAX_CHANNELS; channel++)
   {
        if (NoError == D2K_AI_VoltScale(card, AD_B_10_V,
                        (U16)pRawSampleBuffer[channel], &ADCVoltages[channel]))
        {
            printf("ADCVoltages %f\n",ADCVoltages[channel]);
        }
    }
}

main()
{
    I16 card, err, Id,Id2;
    int card_num, card_type=DAQ_2010;
    unsigned int channel;
    BOOLEAN halfReady, fStop=0, fok=0;    
    BOOLEAN mBufferToReadFrom = FALSE;
    unsigned int channelCount = MAX_CHANNELS;
    
    unsigned short ai_buf[SCANCOUNT*MAX_CHANNELS];
    unsigned short ai_buf2[SCANCOUNT*MAX_CHANNELS];
    unsigned short channelsToRead[MAX_CHANNELS];

    printf("This program inputs %d scans from CH-0 in %d Hz with re-trigger operation.\n", SCANCOUNT, (int)(BASETIME/SCAN_INTERVAL));
    
    printf("--------------------------------------------\n");
    printf("0. DAQ-2005\n1. DAQ-2006\n2. DAQ-2010\n3. DAQ-2016\n4. DAQ-2204\n5. DAQ-2205\n6. DAQ-2206\n7. DAQ-2208\n8. DAQ-2213\n9. DAQ-2214\n10. DAQ-2501\n11. DAQ-2502\n");
    printf("--------------------------------------------\n");
    printf("Please select a card type: ");
    scanf(" %d", &card_type);
    card_type = CardType[card_type];
    printf("Please input a card number: ");
    scanf(" %d", &card_num);
    if ((card=D2K_Register_Card (card_type, (I16)card_num)) <0 ) {
			printf("Register_Card error=%d", card);
			return -1;
    }
  
    for( channel = 0; channel < channelCount; channel++)
    {
        // channels to obtain
        channelsToRead[channel] = channel;
        // set voltage range of ech channel
        err = D2K_AI_CH_Config (card, channel, AIRANGE);
        if (err!=NoError)
        {
           printf("D2K_AI_CH_Config error=%d", err);
           exit(1);
        }
    }  
    err =D2K_AI_Config(card, DAQ2K_AI_ADCONVSRC_Int, DAQ2K_AI_TRGMOD_POST  | DAQ2K_AI_TRGSRC_SOFT, 0, 0, 0, 1);
    if (err!=0)
    {
       printf("D2K_AI_Config error=%d", err);
       exit(1);
    }
    
    err=D2K_AI_AsyncDblBufferMode (card, 1);
    if (err!=NoError)
    {
       printf("D2K_AI_AsyncDblBufferMode error=%d", err);
       exit(1);
    }
	
    err=D2K_AI_ContBufferSetup (card, ai_buf, SCANCOUNT*MAX_CHANNELS, &Id);
    if (err!=0)
    {
       printf("D2K_AI_ContBufferSetup error=%d", err);
       exit(1);
    }
	
    err=D2K_AI_ContBufferSetup (card, ai_buf2, SCANCOUNT*MAX_CHANNELS, &Id2);
    if (err!=0)
    {
       printf("D2K_AI_ContBufferSetup2 error=%d", err);
       exit(1);
    }
	
    err =  D2K_AI_ContReadMultiChannels(card, channelCount, channelsToRead, Id, 2, 640, 160, ASYNCH_OP);
    if (err!=0)
    {
       printf("D2K_AI_ContReadChannel error=%d", err);
       exit(1);
    }

    while(1)
    {       
        D2K_AI_AsyncDblBufferHalfReady(card, &halfReady, &fStop);
        if(halfReady)
        {      
            unsigned int sample = 1;
            if (mBufferToReadFrom == 0)
            {
                //Buffer 0 Half Ready
                mBufferToReadFrom = 1;            
                RetrieveVoltagesFromDoubleBuffer(card, ai_buf );               
            }        
            else
            {
                //Buffer 1 Half Ready
                mBufferToReadFrom = 0;             
                RetrieveVoltagesFromDoubleBuffer(card, ai_buf2);     
            } 
            //Let driver know that ready buffer is handled
            D2K_AI_AsyncDblBufferHandled(card);
        }       
    }   
}
