#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER 0
#define AIRANGE       AD_B_10_V
#define BASETIME      40000000
#define SCANCOUNT     10000
#define ADCONVERTSRC  DAQ2K_AI_ADCONVSRC_Int
#define ADTRIGSRC     DAQ2K_AI_TRGSRC_ANA
#define ADTRIGMODE    DAQ2K_AI_TRGMOD_POST
#define ADTRIGPOL     DAQ2K_AI_TrgPositive
#define ReTrigCOUNT	   0
#define BUFAUTORESET   1
#define SCAN_INTERVAL  400
#define POSTCOUNT      0

#define CardNum   12

U16 CardType[CardNum] = {DAQ_2005, DAQ_2006, DAQ_2010, DAQ_2016, DAQ_2204, DAQ_2205, DAQ_2206, DAQ_2208, DAQ_2213, DAQ_2214, DAQ_2501, DAQ_2502};

I16 card = -1;
short ai_buf[10000];
short ai_buf2[10000];

void show_channel_data(U16 *buf, U32 count, U16 startPos);

int main( void )
{
    I16 err, Id; 
    int card_num, card_type=DAQ_2010;
    BOOLEAN trigready, fStop=0;
    U32 count=0, startPos=0, times=0;
    U16 RdyTrigCnt=0;

    setbuf( stdout, NULL );
    clrscr();

    printf("This program inputs %d scans from CH-0 in %d Hz.\n", SCANCOUNT, (int)(BASETIME/SCAN_INTERVAL));
    printf("--------------------------------------------\n");
    printf("0. DAQ-2005\n1. DAQ-2006\n2. DAQ-2010\n3. DAQ-2016\n4. DAQ-2204\n5. DAQ-2205\n6. DAQ-2206\n7. DAQ-2208\n8. DAQ-2213\n9. DAQ-2214\n10. DAQ-2501\n11. DAQ-2502\n");
    printf("--------------------------------------------\n");
    printf("Please select a card type: ");
    scanf(" %d", &card_type);
    card_type = CardType[card_type];
    printf("Please input a card number: ");
    scanf(" %d", &card_num);
    if ((card=D2K_Register_Card (card_type, (I16)card_num)) <0 ) {
			printf("Register_Card error=%d", card);
			return -1;
    }
    err = D2K_AI_CH_Config (card, CHANNELNUMBER, AIRANGE);
    if (err != NoError) {
			printf("D2K_AI_CH_Config error=%d", err);
			D2K_Release_Card(card);
			return -1;
    }
    err = D2K_AIO_Config (card, DAQ2K_IntTimeBase, Above_High_Level|CH0ATRIG, 0x80, 0x80);
    err = D2K_AI_Config (card, ADCONVERTSRC, ADTRIGSRC|ADTRIGMODE|DAQ2K_AI_ReTrigEn, POSTCOUNT, 0, ReTrigCOUNT, BUFAUTORESET);
    if (err != NoError) {
			printf("D2K_AI_Config error=%d", err);
			D2K_Release_Card(card);
			return -1;
    }

    err=D2K_AI_ContBufferSetup (card, ai_buf, SCANCOUNT,&Id);
    if (err != NoError) {
			printf("D2K_AI_ContBufferSetup error=%d", err);
			D2K_Release_Card(card);
			return -1;
    }
    err=D2K_AI_ContBufferSetup (card, ai_buf2, SCANCOUNT, &Id);
    if (err!=0) {
       printf("D2K_AI_ContBufferSetup2 error=%d", err);
       exit(1);
    }

    printf("\n\n\nStart Data Conversion by External Analog Trigger Signal.\nThe times of Re-TRIGGER is %d.\n", ReTrigCOUNT);
    err = D2K_AI_ContReadChannel (card, CHANNELNUMBER, Id, SCANCOUNT, SCAN_INTERVAL, SCAN_INTERVAL, ASYNCH_OP);
    if (err != NoError) {
			printf("D2K_AI_ContReadChannel error=%d", err);
			D2K_AI_ContBufferReset(card);
			D2K_Release_Card(card);
			return -1;
    }
    do {       
        err = D2K_AI_AsyncReTrigNextReady (card, &trigready, &fStop,&RdyTrigCnt); 
  	if(trigready){ 
                //if(RdyTrigCnt!=(times%2))
		printf("ready buffer index: %d\n",RdyTrigCnt);      
                times += 1;
        }
    } while(!kb_hit());
    D2K_AI_AsyncClear(card, &startPos, &count);
    //Here to handle the data stored in ready buffer
    printf("\n\nTotal %ld input data.\n", count+SCANCOUNT*times);
    show_channel_data(ai_buf, SCANCOUNT, startPos);
    D2K_Release_Card(card);
    printf("\nPress ENTER to exit the program. "); getchar();

    return 0;
}

void show_channel_data( U16 *buf, U32 count, U16 startPos)
{
    int k;
    U32 i;
    F64 voltage;

    printf(" >>>>>>>>>>>>>>> the first 100 valid scans  <<<<<<<<<<<<<<< \n");
    printf("CH0 :\n");
    for (k=0; k<(count/80) && k<10; k++) {
	for( i=k*80; i<(k*80+80) ; i++ ){
            D2K_AI_VoltScale (card, AIRANGE, ai_buf[i], &voltage);
	    printf(" 0x%04x(%+4.2fV)  ", \
		ai_buf[(i+startPos)%SCANCOUNT]&0xffff, voltage);
	    if(i && ((i%4)==3))
		printf("\n");
	}
	if(k<(count/80-1) && k<9) {
	    printf("Press any key for the continued data...\n");
	    getch();
	}
    }
}
