/*********************************************************************
*
* Example program: 
*    AOsimuUpdate.c
*
* Description: 
*    Output two channels, one point for each channel at a time, immediately 
*
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "conio.h"
#include "d2kdask.h"

#define CHANNELNUMBER All_Channels
#define CHANNELCOUNT  2
#define DAPOLARITY	  DAQ2K_DA_BiPolar
#define DAREFERENCE	  DAQ2K_DA_Int_REF
#define DAREFERENCE_VOLTAGE	  10.0
#define CardNum   8

U16 CardType[CardNum] = {DAQ_2005, DAQ_2006, DAQ_2010, DAQ_2016, DAQ_2204, DAQ_2205, DAQ_2206, DAQ_2214};

int main( void )
{
    I16 card, err; 
    int card_num, card_type;
    F32 out_voltage;
    BOOLEAN bstop = FALSE;
    char cont_input;
    I16 outVal[2];

    setbuf( stdout, NULL );
    clrscr();

    /* For best results, look at the output with a scope. */
    printf("This program outputs data to DA0 and DA1\n");
    printf("This program demonstrates General purpose timer/counter GPTC0\n\n");
    printf("--------------------------------------------\n");
    printf("0. DAQ-2005\n1. DAQ-2006\n2. DAQ-2010\n3. DAQ-2016\n4. DAQ-2204\n5. DAQ-2205\n6. DAQ-2206\n7. DAQ-2214\n");
    printf("--------------------------------------------\n");
    printf("Please select a card type: ");
    scanf(" %d", &card_type);
    card_type = CardType[card_type];
    printf("Please input a card number: ");
    scanf(" %d", &card_num);
    if ((card=D2K_Register_Card (card_type, card_num)) <0 ) {
	printf("Register_Card error=%d", card);
	return -1;
    }
    err = D2K_AO_CH_Config (card, CHANNELNUMBER, DAPOLARITY, DAREFERENCE, (F64)DAREFERENCE_VOLTAGE);
    if (err != NoError) {
	printf("D2K_AO_CH_Config error=%d", err);
	D2K_Release_Card(card);
	return -1;
    }

    printf("\n Please input output Voltage value in [%s]:", "-10v ~ +10v");
    scanf(" %f", &out_voltage );

    do{
        //transform voltage to Hexadecimal value
	outVal[0]= outVal[1]= (U16)( (out_voltage +10) / 20 * 4095 );

	err = D2K_AO_SimuWriteChannel (card, CHANNELCOUNT, outVal);
	if (err != NoError) {
	    printf("D2K_AO_SimuWriteChannel error=%d", err);
	    D2K_Release_Card(card);
	    return -1;
	}

	printf(" DA0/DA1 Output Voltage : %f V (0x%x) ", out_voltage, outVal[0]);
	printf("\n");
	usleep(500000);

	printf("                 press (C)ontinue or Quit :");
	cont_input=getch();

	if( cont_input != 'C' && cont_input != 'c' ){
	    bstop = TRUE;
	}
	else {
	    bstop = FALSE;
	    printf("\n\n Please input output Voltage value in [%s]:", "-10v ~ +10v");
	    scanf(" %f", &out_voltage );
	}
    }while( !bstop );

    D2K_Release_Card(card);

    return 0;

}

