#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <math.h>
#include <sys/time.h>
#include <sys/mman.h>
#include "d2kdask.h"

U16 channel=3;  //4 channels
U16 chans[4]={0,1,2,3};
U16 range=AD_B_10_V;
char *file_name="dmaData.txt";
U32 read_count=40000;
U16 ai_buf[40000];
I16 card = -1, multi = 0;
FILE* fp;

struct timeval tv_start, tv_end;
struct timezone tz;


#pragma pack(push,1)

typedef struct _D2K_MODULE_PARAM
{
		short	card_number;			
		short	card_type;	
		unsigned short ch;
		unsigned long   scancount;
		unsigned long   scan_interval;
}
D2K_MODULE_PARAM;

#pragma pack(pop)

D2K_MODULE_PARAM param;

void write_buffer_to_file( void );

#if 0
void signal_handler(int sig_no)
{
    U32 startPos = 0, count1=0;
    I16 err = 0;

    err = D2K_AI_AsyncClear(card, &startPos, &count1);
    if( err != 0 ){
        printf("D2K_AI_AsyncClear Failed !!!, Ready to system hang !!!\n" );
        getch();
    }
    printf("Press any key to save the acquisition data\n");
}
#endif

void evt_handler(int type, void* arg)
{  
    U32 startPos = 0, count1=0;
    I16 err = 0;

   gettimeofday( &tv_end, &tz );
   printf(" End time for DMA operation is: %ld.%06ld sec\n", tv_end.tv_sec, tv_end.tv_usec);

  printf("end: evt_%hd|cardtype_%hd|cardid_%hd\n", type, ((D2K_MODULE_PARAM *) arg)->card_type, ((D2K_MODULE_PARAM *) arg)->card_number);

    err = D2K_AI_AsyncClear(card, &startPos, &count1);
    if( err != 0 ){
        printf("D2K_AI_AsyncClear Failed !!!, Ready to system hang !!!\n" );
        getch();
    }
    printf("Press any key to save the acquisition data\n");

}

#define CardNum   12

U16 CardType[CardNum] = {DAQ_2005, DAQ_2006, DAQ_2010, DAQ_2016, DAQ_2204, DAQ_2205, DAQ_2206, DAQ_2208, DAQ_2213, DAQ_2214, DAQ_2501, DAQ_2502};

int main( void )
{
    I16 err, i, Id;
    int card_num = 0, card_type=DAQ_2010;

    if( (fp = fopen( file_name, "w+b" ) ) == NULL )
	return -10;

    fprintf( fp, " ContScanChannels with Single Buffer Transfer \n\n\n");
    fprintf( fp, "CH0   CH1   CH2   CH3 \n======================\n");

    clrscr();
    setbuf( stdout, NULL );
   tz.tz_minuteswest = 0;
   tz.tz_dsttime = 0;

    printf("This program inputs %ld data from CH-0 to CH-%d of DAQ-2000#%d in %d Hz, and\nstore data to file '%s'.\nPlease press any key to start the operation.\n", read_count, channel, card_num, (int)10000, file_name);

    printf("--------------------------------------------\n");
    printf("0. DAQ-2005\n1. DAQ-2006\n2. DAQ-2010\n3. DAQ-2016\n4. DAQ-2204\n5. DAQ-2205\n6. DAQ-2206\n7. DAQ-2208\n8. DAQ-2213\n9. DAQ-2214\n10. DAQ-2501\n11. DAQ-2502\n");
    printf("--------------------------------------------\n");
    printf("Please select a card type: ");
    scanf(" %d", &card_type);
    if(card_type>3)
	multi = 1;
    else 
        multi = 0;
    card_type = CardType[card_type];
    printf("Please input a card number: ");
    scanf(" %d", &card_num );

    if ((card=D2K_Register_Card (card_type, (I16)card_num)) <0 ) {
        printf("Register_Card error=%d", card);
        return -1;
    }

    for(i=0; i<4; i++)
	D2K_AI_CH_Config (card, i, range);

    D2K_AI_Config (card, 0, 0, 0, 0, 0, 1);

	//set callback param
	param.card_number=card;
	param.card_type=card_type;
	param.ch=(channel+1);
	param.scan_interval=4000;
	param.scancount=read_count/(channel+1);

    err = D2K_AI_ContBufferSetup (card, ai_buf, read_count, &Id);
    if (err!=0) {
       printf("AI_ContBufferSetup error=%d", err);
       D2K_Release_Card( card );
       exit(1);
    }

    // setup signal handler 
    D2K_AI_EventCallBackEx (card, 1, DAQEnd, evt_handler, (void* ) &param);
    gettimeofday( &tv_start, &tz );
    printf(" Start time for DMA operation is: %ld.%06ld sec\n", tv_start.tv_sec, tv_start.tv_usec);
    if(multi)
     err = D2K_AI_ContReadMultiChannels(card, channel+1, chans, Id, read_count/(channel+1), 4000, 4000/(channel+1), ASYNCH_OP);
    else
     err = D2K_AI_ContReadMultiChannels(card, channel+1, chans, Id, read_count/(channel+1), 4000, 4000, ASYNCH_OP);
    if (err!=0) {
       printf("AI_ContScanChannelsToFile error=%d", err);
       D2K_Release_Card( card );
       exit(1);
    }
    printf("Data acquisition ...\n");
    getch();

    write_buffer_to_file();

    // reset signal handler 
    D2K_AI_EventCallBackEx (card, 0, DAQEnd, NULL, NULL);	
    D2K_Release_Card(card);

    fclose( fp );
    printf("\n\nThe input data is already stored in file '%s'.\n", file_name);
    printf("\nPress any key to exit the program. "); 
    getch();
    putchar('\n');
    return 0;
}

void write_buffer_to_file( void )
{
  I16 adinput[4];
  char s[50], temp_s[10];
  U32 i;
  int j;
  int chan_no = channel +1;
  F64 voltage = 0.0;
	
  for(i=0; i< read_count; i+=chan_no){
    	s[0] = 0x00;
    	for( j = 0; j < chan_no; j++ ){
      	   //printf( "%x  ", (F32)(ai_buf[i+j]) );
           D2K_AI_VoltScale (card, range, ai_buf[i+j], &voltage);
           sprintf(temp_s, "%4.4f ",  voltage);
      	   strcat( s, temp_s );        
    	}
    	strcat(s, "\n");
    	fwrite(s, sizeof(U8), strlen(s), fp);
 }
}




